import { useFormContext } from 'react-hook-form';
import { AnimatePresence, motion } from 'framer-motion';
import { useState } from 'react';
import { CheckIcon, XMarkIcon } from '@heroicons/react/24/outline';
import FormError from './FormError';

export interface FormInputProps extends Omit<React.InputHTMLAttributes<HTMLInputElement | HTMLTextAreaElement>, 'type'> {
    name: string;
    label: string;
    type?: 'text' | 'textarea' | 'password' | 'email' | 'number' | 'tel' | 'url' | 'search' | 'date' | 'time' | 'datetime-local';
    rows?: number;
}

export function FormInput({
    name,
    label,
    type = 'text',
    rows = 4,
    className,
    ...restProps
}: FormInputProps) {
    const [isFocused, setIsFocused] = useState(false);
    const {
        register,
        formState: { errors },
        watch,
        setValue,
        trigger, // Add trigger from useFormContext
    } = useFormContext();
    const handleBlur = async () => {
        setIsFocused(false);
        await trigger(name); // Trigger validation for this field
    };
    const value = watch(name);
    const error = errors[name]?.message as string;
    const isValid = value && !error;

    const inputClasses = `
    w-full rounded-md border bg-white p-2 pr-8 outline-none shadow-sm
    transition-all duration-300 ease-out placeholder:text-gray-400
    ${error ? 'border-red-500 focus:border-red-500 focus:ring-red-200' : 'border-gray-300 focus:border-blue-500 focus:ring-blue-200'}
    ${isFocused ? 'ring-2 ring-opacity-50' : ''}
    ${className || ''}
  `;

    const InputElement = type === 'textarea' ? 'textarea' : 'input';

    return (
        <div className="space-y-2">
            <div className="flex justify-between">
                <label className="block text-sm font-medium text-gray-700">{label}</label>
                {restProps.maxLength && (
                    <span className="text-sm text-gray-500">
                        {value?.length || 0}/{restProps.maxLength}
                    </span>
                )}
            </div>

            <div className="relative">
                <InputElement
                    {...register(name)}
                    type={type !== 'textarea' ? type : undefined}
                    rows={type === 'textarea' ? rows : undefined}
                    {...restProps}

                    onFocus={() => setIsFocused(true)}
                    onBlur={handleBlur}
                    className={inputClasses}
                />

                <div className="absolute right-2 top-1/2 -translate-y-1/2 flex items-center space-x-1">
                    {value && isFocused && (
                        <button
                            type="button"
                            className="p-1 text-gray-400 hover:text-gray-600 transition-colors"
                            onMouseDown={(e) => e.preventDefault()}
                            onClick={() => setValue(name, '')}
                        >
                            <XMarkIcon className="w-4 h-4" />
                        </button>
                    )}
                    {isValid && <CheckIcon className="text-green-500 w-4 h-4" />}
                </div>
                <FormError error={error}></FormError>
            </div>
        </div>
    );
}